#define MODULE "dir"

#include "luna/dirent.h"
#include "fs/VFS.h"
#include "interrupts/Context.h"
#include "std/errno.h"
#include "std/string.h"
#include "sys/UserMemory.h"
#include "thread/Scheduler.h"

void sys_getdents(Context* context, int fd, struct luna_dirent* buf, size_t count)
{
    if (fd < 0 || fd > TASK_MAX_FDS)
    {
        context->rax = -EBADF;
        return;
    }
    Task* current_task = Scheduler::current_task();
    if (!current_task->files[fd].is_open())
    {
        context->rax = -EBADF;
        return;
    }
    Descriptor& dir = current_task->files[fd];
    VFS::Node* node = dir.node();
    if (node->type != VFS_DIRECTORY)
    {
        context->rax = -ENOTDIR;
        return;
    }
    size_t nread = 0;
    while (count)
    {
        VFS::Node* entry = VFS::readdir(node, dir.offset());
        if (!entry)
        {
            context->rax = nread;
            return;
        }

        luna_dirent dirent;

        dirent.total = node->length;
        dirent.offset = dir.offset();
        dirent.inode = entry->inode;
        strlcpy(dirent.name, entry->name, sizeof(dirent.name));

        if (!copy_typed_to_user(buf, &dirent))
        {
            context->rax = -EFAULT;
            return;
        }

        dir.seek(dir.offset() + 1);
        buf++;
        nread++;
        count--;
    }
    context->rax = nread;
}